using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace Stencils
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        private Texture2D cbxTexture;
        private Texture2D maskTexture;
        private Rectangle cbxRectangle;
        private Rectangle maskRectangle;
        private bool settingMask = true;
        private StencilHelper stencils;
        private SpriteFont font;


        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            this.maskRectangle = new Rectangle(200, 200, 500, 500);
            this.cbxRectangle = new Rectangle(300, 300, 256, 256);

            this.IsMouseVisible = true;
            graphics.PreferredBackBufferWidth = 1280;
            graphics.PreferredBackBufferHeight = 720;
            graphics.PreferredDepthStencilFormat = SelectStencilMode();
            graphics.ApplyChanges();


        }

        private DepthFormat SelectStencilMode()
        {
            // Check stencil formats
            GraphicsAdapter adapter = GraphicsAdapter.DefaultAdapter;
            SurfaceFormat format = adapter.CurrentDisplayMode.Format;
            if (adapter.CheckDepthStencilMatch(DeviceType.Hardware, format, format, DepthFormat.Depth24Stencil8))
                return DepthFormat.Depth24Stencil8;
            else if (adapter.CheckDepthStencilMatch(DeviceType.Hardware, format, format, DepthFormat.Depth24Stencil8Single))
                return DepthFormat.Depth24Stencil8Single;
            else if (adapter.CheckDepthStencilMatch(DeviceType.Hardware, format, format, DepthFormat.Depth24Stencil4))
                return DepthFormat.Depth24Stencil4;
            else if (adapter.CheckDepthStencilMatch(DeviceType.Hardware, format, format, DepthFormat.Depth15Stencil1))
                return DepthFormat.Depth15Stencil1;
            else
                throw new ApplicationException("Could not find stencil buffer for default adapter");
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            base.Initialize();

            this.stencils = new StencilHelper(this.GraphicsDevice);
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            this.cbxTexture = this.Content.Load<Texture2D>("CBX256x256");
            this.maskTexture = this.Content.Load<Texture2D>("White1x1");
            this.font = this.Content.Load<SpriteFont>("CourierNew");
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            var mouse = Mouse.GetState();
            if (mouse.LeftButton == ButtonState.Pressed)
            {
                if (this.settingMask)
                {
                    this.maskRectangle.Location = new Point(mouse.X, mouse.Y);
                }
                else
                {
                    this.cbxRectangle.Location = new Point(mouse.X, mouse.Y);
                }
            }

            if (mouse.RightButton == ButtonState.Pressed)
            {
                if (this.settingMask)
                {
                    this.maskRectangle.Width = mouse.X - this.maskRectangle.X;
                    this.maskRectangle.Height = mouse.Y - this.maskRectangle.Y;
                }
                else
                {
                    this.cbxRectangle.Width = mouse.X - this.cbxRectangle.X;
                    this.cbxRectangle.Height = mouse.Y - this.cbxRectangle.Y;
                }
            }

            var kb = Keyboard.GetState();
            if (kb.IsKeyDown(Keys.M)) this.settingMask = true;
            if (kb.IsKeyDown(Keys.T)) this.settingMask = false;
            if (kb.IsKeyDown(Keys.Escape)) this.Exit();


            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            // Render the area where the stenciling will occour so we can see it
            this.spriteBatch.Begin();
            this.spriteBatch.Draw(this.maskTexture, this.maskRectangle, Color.LightGray);
            this.spriteBatch.End();

            // begin the stencil render
            stencils.Clear();
            stencils.Begin();

            // render the stencil
            this.spriteBatch.Begin();
            this.spriteBatch.Draw(this.maskTexture, this.maskRectangle, Color.White);
            this.spriteBatch.End();

            // done rendering stencil
            stencils.End();

            // render the image that will be stenciled
            this.spriteBatch.Begin();
            this.spriteBatch.Draw(this.cbxTexture, this.cbxRectangle, Color.White);
            this.spriteBatch.End();

            // complete the stencil process (disables stenciling)
            stencils.Done();

            // draw info
            this.spriteBatch.Begin();
            this.spriteBatch.DrawString(this.font, "Use left and right mouse buttons to set position\r\n" +
                                                   "Press T to set texture position\r\n" +
                                                   "Pres M to set mask position" , Vector2.Zero, Color.White);
            this.spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
