using Microsoft.Xna.Framework.Graphics;

namespace Stencils
{
    public class StencilHelper
    {
        private GraphicsDevice device;
        private RenderState state;
        private int stage = 0;

        public StencilHelper(GraphicsDevice device)
        {
            this.device = device;
            this.state = this.device.RenderState;
        }

        public void Clear()
        {
            //Clear the stencil buffer
            this.device.Clear(ClearOptions.Stencil, Color.Black, 0, 0);
        }

        public void Begin()
        {
            if (this.stage != 0) return;

            //Configure the stencil buffer to mask out the area we want to use as the mirror
            this.state.StencilEnable = true;
            this.state.ColorWriteChannels = ColorWriteChannels.None;
            this.state.ReferenceStencil = 0;
            this.state.StencilFunction = CompareFunction.Equal;

            //IncrementSaturation prevents the stencil increment from rolling the stencil value over
            //back to zero if it increases past the maximum bit precision of the stencil buffer
            this.state.StencilPass = StencilOperation.IncrementSaturation;
            this.state.DepthBufferWriteEnable = false;
            this.stage = 1;
        }

        public void End()
        {
            if (this.stage != 1) return;
            //Configure the stencil buffer to render anything behind the current mask            
            this.state.DepthBufferWriteEnable = true;
            this.state.ColorWriteChannels = ColorWriteChannels.All;
            this.state.ReferenceStencil = 1;
            this.state.StencilPass = StencilOperation.Keep;
            this.state.StencilFunction = CompareFunction.LessEqual;
            this.stage = 2;
        }

        public void Done()
        {
            if (this.stage != 2) return;

            //Reset the cull mode and disable the stencil buffer
            this.state.StencilEnable = false;
            this.stage = 0;
        }
    }
}